//  
//  Copyright (C) 2009 Bartek thindil Jasicki
// 
//  This file is part of Grubng
// 
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 

using System;
using System.Text;

namespace urlsdb
{
	
	/// <summary>
	/// Class provide functions for manipulate URL's
	/// </summary>
	internal class ParseURLs
	{
		System.Security.Cryptography.SHA1Managed SHhash;
		StringBuilder key;
		byte[] HashValue;
		byte[] PureHash;
		
		/// <summary>
		/// Standard class constructor.
		/// </summary>
		public ParseURLs()
		{
			this.SHhash = new System.Security.Cryptography.SHA1Managed();
			this.key = new StringBuilder();
		}
		
		/// <summary>
		/// Function manipulate URL - check it correctness, drop to lower case host.
		/// </summary>
		/// <param name="url">
		/// A <see cref="System.String"/> URL to parse.
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/> parsed URL or String.Empty if URL is invalid.
		/// </returns>
		public string ParseURL(string url)
		{
			//Remove http(s):// from URL
			int index = url.IndexOf("//");
			if (index > 0)
			{
				url = url.Remove(0, (index + 2));
			}
			//Check correctness of URL
			if (!System.Text.RegularExpressions.Regex.IsMatch(url, 
			                                                  @"^[a-z0-9]+([a-z0-9\=\-\.\?\,\'\/\\\+&amp;%\$#_~]*)?$",
			                                                  System.Text.RegularExpressions.RegexOptions.IgnoreCase))
			{
				return String.Empty;
			}
			//Conver to lower case host name
			index = url.IndexOf('/');
			string host, path;
			if (index > 0)
			{
				host = url.Substring(0, index);
				path = url.Substring(index);
			}
			else
			{
				host = url;
				path = "/";
			}
			host = host.ToLower();
			//Validate hostname
			if (!System.Text.RegularExpressions.Regex.IsMatch(url, @"^[a-z0-9]+([a-z0-9\-\.]*)?$",
			                                                  System.Text.RegularExpressions.RegexOptions.IgnoreCase))
			{
				return String.Empty;
			}
			int count = 0;
			index = 0;
			while (index != -1)
			{
				index = host.IndexOf('.', index);
				if (index > -1)
				{
					count ++;
					index ++;
				}
			}
			if (count == 1)
			{
				host = "www." + host;
			}
			url = host + path;
			url = System.Web.HttpUtility.UrlEncode(host + path);
			url = url.Replace("%2f", "/");
			url = url.Replace("%2c", ",");
			url = url.Replace("%3d", "=");
			url = url.Replace("%3f", "?");
			url = url.Replace("%26", "&");
			url = url.Replace("%3b", ";");
			url = url.Replace("%3a", ":");
			url = url.Replace("%40", "@");
			url = url.Replace("%23", "#");
			url = url.Replace("%7e", "~");
			url = url.Replace("%252f", "/");
			return url;
		}
		
		/// <summary>
		/// Function return SHA1 hash of URL
		/// </summary>
		/// <param name="url">
		/// A <see cref="System.String"/> URL to hash
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/> SHA1 hash of URL
		/// </returns>
		public string GetHash(string url)
		{
			if (this.key.Length > 0)
			{
				this.key.Remove(0, key.Length);
			}
			this.PureHash = Encoding.UTF8.GetBytes(url);
			this.HashValue = SHhash.ComputeHash(this.PureHash);
			foreach(byte b in this.HashValue) 
			{
				this.key.Append(String.Format("{0:x2}", b));
			}
			return this.key.ToString();
		}
	}
}
